<?php
/**
 * localfile - class for manipulation of local file with locking
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Haruki Setoyama <haruki@planewave.org>
 * @copyright Copyright &copy; 2004, Haruki SETOYAMA <haruki@planewave.org>
 * @license http://www.opensource.org/licenses/lgpl-license.php LGPL
 * @version $Id: localfile.php,v 1.4 2004/10/19 03:55:40 haruki Exp $
 * @package localfile
 */
/**
 * Requires PEAR
 */
require_once 'PEAR.php';
/**
 * error code
 */
define('LOCALFILE_ERROR', 1);
/**
 * type of file lock
 */
@define('LOCALFILE_LOCKTYPE', 'Flock');
/**
 * mode for writable file and directory
 * If user of PHP executable differs of you,
 * to unlink files this class create, file/dir modes must be set like below.
 * But it may be little bit unsecure in certain circumstances.
 */
@define('LOCALFILE_FILE_MODE', 0666);
@define('LOCALFILE_DIR_MODE', 0777);

/**
 * localfile - class for manipulation of local file with locking
 *
 * @access public
 * @package localfile
 */
class LocalFile {
    function create($file)
    {
        $class = 'LocalFile_'.LOCALFILE_LOCKTYPE;
        return new $class($file);
    }
}

/**
 * LocalFile_Flock - class for manipulation of local file with flock
 *
 * @package localfile
 */
class LocalFile_Flock extends PEAR {

    /**
     * flag for use of flock().
     */
    var $flock = true;

    /**
     * file pointer for lock file
     * @access private
     */
    var $_lfp;

    /**
     * file name to be maniputated
     * @access private
     */
    var $_file;

    /**
     * if file_get_contents() function exists or not
     * @access private
     */
    var $_file_get_contents;
    
    /**
     * error objects
     * @access private
     */
    var $_error_class = 'LocalFile_Error';

    /**
     * constructor
     * @param string $file
     */
    function LocalFile_Flock($file)
    {
        $this->_file = $file;
        $this->_file_get_contents = function_exists('file_get_contents');
    }

    /**
     * read content of the file
     * @return string|PEARError
     */
    function read($new = false)
    {
        $lock = $this->lock(LOCK_SH);
        if ($lock !== true) { return $err; }

        if ($new || ! file_exists($this->_file)) {
            $ret = touch($this->_file);
            if ($ret === false) {
                $msg = "Failed to create the file '$this->_file': $php_errormsg. ";
                $this->unlock();
                return $this->raiseError($msg);
            }
            return '';
        }

        if ($this->_file_get_contents) {
            $content = file_get_contents($this->_file);
        } else {
            $fp = fopen($this->_file, 'rb');
            if ($fp === false) {
                $msg = "Failed to open the file '$this->_file' for reading: $php_errormsg. ";
                $this->unlock();
                return $this->raiseError($msg);
            }
            $content = fread($fp, filesize($this->_file));
        }
        if ($content === false) {
            $msg = "Failed to get content from the file '$this->_file': $php_errormsg. ";
            $this->unlock();
            return $this->raiseError($msg);
        }
        $this->unlock();
        return $content;
    }

    /**
     * write $content to the file, and unlock the file
     * @param string
     */
    function write($content)
    {
        // file lock.
        $lock = $this->lock();
        if ($lock !== true) { return $lock; }

        $tempf = tempnam(dirname($this->_file), 'tmp.');
        $fp_w = $this->_fopen_wb($tempf);
        if (PEAR::isError($fp_w)) return $fp_w;
        $err = fwrite($fp_w, $content);
        if ($err === false) {
            $msg = "Failed to write the content to the file '$tempf': $php_errormsg. ";
            fclose($fp_w);
            unlink($tempf);
            $this->unlock();
            return $this->raiseError($msg);
        }
        fclose($fp_w);

        if (file_exists($this->_file)) {
            $err = unlink($this->_file);
            if ($err === false) {
                $msg = "Failed to unlink old file '$this->_file': $php_errormsg. ";
                unlink($tempf);
                $this->unlock();
                return $this->raiseError($msg);
            }
        }
        $err = rename($tempf, $this->_file);
        if ($err === false) {
            $msg = "Failed to rename the temporary file to '$this->_file': $php_errormsg. ";
            unlink($tempf);
            $this->unlock();
            return $this->raiseError($msg);
        }
        $this->unlock();
        chmod($this->_file, LOCALFILE_FILE_MODE);
        return true;
    }

    /**
     * lock file
     */
    function lock($type = LOCK_EX)
    {
        if (isset($this->_lfp)) {
            return true;
        }
        $lock = $this->_file.'.lock';
        $ret = $this->_fopen_wb($lock);
        if (PEAR::isError($ret)) return $ret;
        $this->_lfp = $ret;
        if (! flock($this->_lfp, $type)) {
            $msg = "Failed to flock '$lock': $php_errormsg. ";
            fclose($this->_lfp);
            unset($this->_lfp);
            return $this->raiseError($msg);
        }
        return true;
    }

    /**
     * unlock  file
     */
    function unlock()
    {
        if (isset($this->_lfp)) {
            $lock = $this->_file.'.lock';
            fclose($this->_lfp);
            $err = unlink($lock);
            if ($err === false) {
                $msg = "Failed to unlink the lock file '$lock': $php_errormsg. ";
                return $this->raiseError($msg);
            }
        }
        return true;
    }

    /**
     * fopens file with 'wb' mode.
     * if no dir for the file, mkdirs the dirs recursively.
     * @param string $file
     * @return true|PEARError
     * @access private
     */
    function _fopen_wb($file)
    {
        if (! file_exists($file)) {
            $ret = $this->_mkdirs(dirname($file));
            if ($ret !== true) return $ret;
        }
        
        $fp = fopen($file, 'wb');
        if ($fp === false) {
            $msg = "Failed to fopen '$file' for writing: $php_errormsg. ";
            return $this->raiseError($msg);
        }
        return $fp;
    }

    /**
     * makes writable directory recursively
     *
     * @param stirng $path
     * @return true|PEARError
     * @access private
     */
    function _mkdirs($path)
    {
        if (is_dir($path)) return true;
        if (is_file($path)) {
            return $this->raiseError("'$path' is not directory. ");
        }
        $ret = $this->_mkdirs(dirname($path));
        if ($ret !== true) return $ret;
        $mask = umask((LOCALFILE_DIR_MODE or 0600) xor 0777);
        $ret = mkdir($path, 0777);
        if ($ret !== true) {
            $msg = "Failed to mkdir '$path': $php_errormsg. ";
            umask($mask);
            return $this->raiseError($msg);
        }
        umask($mask);
        return true;
    }
}

/**
 * Localfile_Error
 *
 * @package localfile
 */
class LocalFile_Error extends PEAR_Error 
{
    function LocalFile_Error($message = 'unknown error', $code = null,
                         $mode = null, $options = null, $userinfo = null)
    {
        if ($code === null) {
            $code = LOCALFILE_ERROR;
        }
        $this->PEAR_Error("LocalFile Error: $message\n", $code, $mode, $options, $userinfo);
    }
}
?>